package com.qmth.teachcloud.mark.lock.impl;

import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockProvider;
import org.redisson.api.RReadWriteLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * 自己实现的内存锁控制
 *
 * @author luoshi
 */
@Component("customRedisLockProvider")
public class CustomRedisLockProvider implements LockProvider {

    @Resource
    RedisUtil redisUtil;

    @Resource
    RedissonClient redissonClient;

    @Override
    public void waitLock(LockType type, String key) {
        getLock(type, key).writeLock().lock(SystemConstant.REDIS_LOCK_DEFAULT_TIME_OUT, TimeUnit.SECONDS);
    }

    @Override
    public boolean tryLock(LockType type, String key) {
        try {
            return getLock(type, key).writeLock().tryLock(30L, SystemConstant.REDIS_LOCK_DEFAULT_TIME_OUT, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return false;
    }

    @Override
    public boolean isLocked(LockType type, String key) {
        return getLock(type, key).writeLock().isLocked();
    }

    @Override
    public void unlock(LockType type, String key) {
        getLock(type, key).writeLock().unlock();
    }

    @Override
    public void watch(LockType type, String key) {
        getLock(type, key).readLock().lock(SystemConstant.REDIS_LOCK_DEFAULT_TIME_OUT, TimeUnit.SECONDS);
    }

    @Override
    public void unwatch(LockType type, String key) {
        getLock(type, key).readLock().unlock();
    }

    private RReadWriteLock getLock(LockType type, String key) {
        return redissonClient.getReadWriteLock(type.name() + ":" + key);
    }

    @Override
    public void clear() {
        LockType[] lockTypes = LockType.values();
        for (LockType lockType : lockTypes) {
            Set<String> keySet = (Set<String>) redisUtil.getKeyPatterns(lockType.name() + ":*");
            if (!CollectionUtils.isEmpty(keySet)) {
                for (String s : keySet) {
                    redisUtil.delete(s);
                }
            }
        }
    }
}
